unit interp;

interface

uses
    int_stek, int_def;


type

    {Opys obyektu interpretatora}
     TInterpProg=object
        platformDos:boolean;
        zminniFirst:TPZminnaZapys; {Vkazivnyk na spysok zminnyh}
        proceduryFirst:TPProcedureZapys; {Vkazivnyk na spysok procedur}
        fatalError:boolean;        {pokaznyk RUNTIME ERROR}
        errorStr:string;           {tekst pomylky}
        f:text;                    {tymchasovo fail dlia roboty z skryptom programy}
        isRunning:boolean;         {pokazuye chy programa vykonuyetsia}
        isOutput:boolean;
        glybynaPidprogram:integer; {pokazuye yaka iteraciya pidprogramy}
        potochnyiOperator:longint; {nomer potochnogo operatora}
        isDeclarationProcedure:boolean;     {chy rejym deklaraciyi}
        returnToStek:TPLongintStek;
        waitingForInput:boolean;   {  ⠭ ० i㢠}
        resultInput:string;
        osInputVarName:string;
       {Slujbovi funkciyi}
        procedure InitSpysokZminnyh;
        procedure KillSpysokZminnyh;
      {Aryfmetychni vyrazy}
        function ObchyslytyString(param:string):string;
        function ObchyslytyInt(param:string):integer;
        function ObchyslytyReal(param:string):Real;
        function ObchyslytyBool(param:string):integer;
       {Dostup do zminnyh}
        function VarDeclared(name:string):boolean;
        function GetType(name:TIdentyfikator):TTypString;
        function GetAsPointer(name:string):pointer;
        function GetAsInt(name:string):integer;
        function GetAsReal(name:TIdentyfikator):real;
        function GetAsString(name:TIdentyfikator):string;
        procedure SetZminna(name:TIdentyfikator; param:string);
        procedure SetZminnaAsPointer(name:TIdentyfikator; param:pointer);
        procedure AddVar(imya:TIdentyfikator; typ:string);
        procedure AddVarLink(imya:TIdentyfikator; typ:string; linkimya:TIdentyfikator);
        procedure KillVar(name:TIdentyfikator);
        procedure AddProcedure(name:string;params:string);
        function GetProcParameters(name:string):string;
        function GetProcLine(name:string):integer;
       {Rizni}
        function GetTypVyrazu(name:string):TTypString;
       {Operatory}
        procedure OperatorLet(param:string);
        procedure OperatorVar(param:string);
        procedure OperatorVarClass(param:string);
        procedure OperatorKillVarClass(param:string);
        procedure OperatorPrintVal(param:string; lin:boolean);
        procedure OperatorInput(param:string);
        procedure EndOfOperatorInput;
        procedure OperatorKillVar(param:string);
        procedure OperatorGoto(param:string);
        procedure OperatorProcedure(param:string);
        procedure OperatorGosub(param:string);
        procedure OperatorReturn(param:string);
        procedure OperatorIf(param:string);
        procedure ExecOperator(param:string);
       {Inshe}
        procedure Run(s:string);
        procedure OutPutError;
        procedure InitExecFile(filename:string);
        procedure NextOperator;
        procedure ContinueAfterInput(inputVal:string);
      end;


implementation

uses
    int_stpr, int_plt, sysutils;

{
Postfixnyi polskiy zapys vydu x y * 2 -
}
function TInterpProg.ObchyslytyInt(param:string):integer;
var
   st:TPIntStek;
   s:string;
   i1, i2:integer;
   i, kilkist:integer;
   z1,z2:integer;
begin
   st:=nil;
   if pos('[',param)>0 then {Obchyslennia vlojenyh indeksiv masyvu}
     begin
       kilkist:=0;
       i:=0;
       while i<length(param) do
         begin
           inc(i);
           if param[i]='[' then
             begin
               inc(kilkist);
               if kilkist=1 then
                  z1:=i;
             end;
           if param[i]=']' then
             begin
               dec(kilkist);
               if kilkist=0 then
                 begin
                   z2:=i;
                   s:=copy(param, z1+1, z2-z1-1);
                   delete(param, z1+1, z2-z1-1);
                   s:=IntToStr(ObchyslytyInt(s));
                   insert(s, param, z1+1);
                   i:=z1+1;
                   while param[i]<>']' do inc(i);
                 end;
             end;
         end;
     end;
   while param<>'' do
     begin
       if pos(' ',param)=0 then
         begin
           s:=param;
           param:='';
         end
       else
         begin
           s:=copy(param, 1, pos(' ',param)-1);
           delete(param, 1, pos(' ',param));
         end;
       if s='+' then
         begin
           i1:=PopIntStek(st);
           i2:=PopIntStek(st);
           PushIntStek(st, i1+i2);
         end
       else if s='*' then
         begin
           i1:=PopIntStek(st);
           i2:=PopIntStek(st);
           PushIntStek(st, i1*i2);
         end
       else if s='/' then
         begin
           i2:=PopIntStek(st);
           i1:=PopIntStek(st);
           if i2=0 then
             begin
               fatalError:=true;
               errorStr:='Dilennia na nul (division by zero)';
               PushIntStek(st, 0);
             end
           else
             PushIntStek(st, i1 div i2);
         end
       else if s='-' then
         begin
           i2:=PopIntStek(st);
           i1:=PopIntStek(st);
           PushIntStek(st, i1-i2);
         end
       else if s='and' then
         begin
           i1:=PopIntStek(st);
           i2:=PopIntStek(st);
           if (i1>0) and (i2>0) then
              PushIntStek(st, 1)
           else
              PushIntStek(st, 0);
         end
       else if s='or' then
         begin
           i1:=PopIntStek(st);
           i2:=PopIntStek(st);
           if (i1>0) or (i2>0) then
              PushIntStek(st, 1)
           else
              PushIntStek(st, 0);
         end
       else if s='not' then
         begin
           i1:=PopIntStek(st);
           if (i1>0) then
              PushIntStek(st, 0)
           else
              PushIntStek(st, 1);
         end
       else if s='=' then
         begin
           i1:=PopIntStek(st);
           i2:=PopIntStek(st);
           if i1=i2 then
              PushIntStek(st, 1)
           else
              PushIntStek(st, 0);
         end
       else if s='>=' then
         begin
           i2:=PopIntStek(st);
           i1:=PopIntStek(st);
           if i1>=i2 then
              PushIntStek(st, 1)
           else
              PushIntStek(st, 0);
         end
       else if s='<=' then
         begin
           i2:=PopIntStek(st);
           i1:=PopIntStek(st);
           if i1<=i2 then
              PushIntStek(st, 1)
           else
              PushIntStek(st, 0);
         end
       else if s='>' then
         begin
           i2:=PopIntStek(st);
           i1:=PopIntStek(st);
           if i1>i2 then
              PushIntStek(st, 1)
           else
              PushIntStek(st, 0);
         end
       else if s='<' then
         begin
           i2:=PopIntStek(st);
           i1:=PopIntStek(st);
           if i1<i2 then
              PushIntStek(st, 1)
           else
              PushIntStek(st, 0);
         end
       else if s='<>' then
         begin
           i1:=PopIntStek(st);
           i2:=PopIntStek(st);
           if i1<>i2 then
              PushIntStek(st, 1)
           else
              PushIntStek(st, 0);
         end
       else
         begin
           if IsIdentyfikator(s) then
              PushIntStek(st, GetAsInt(s))
           else
              PushIntStek(st, StrToInt(s))
         end;
     end;
   ObchyslytyInt:=PopIntStek(st);
end;

function TInterpProg.ObchyslytyReal(param:string):Real;
var
   st:TPRealStek;
   s:string;
   i1, i2:real;
   i, kilkist:integer;
   z1,z2:integer;
begin
   st:=nil;
   if pos('[',param)>0 then {Obchyslennia vlojenyh indeksiv masyvu}
     begin
       kilkist:=0;
       i:=0;
       while i<length(param) do
         begin
           inc(i);
           if param[i]='[' then
             begin
               inc(kilkist);
               if kilkist=1 then
                  z1:=i;
             end;
           if param[i]=']' then
             begin
               dec(kilkist);
               if kilkist=0 then
                 begin
                   z2:=i;
                   s:=copy(param, z1+1, z2-z1-1);
                   delete(param, z1+1, z2-z1-1);
                   s:=IntToStr(ObchyslytyInt(s));
                   insert(s, param, z1+1);
                   i:=z1+1;
                   while param[i]<>']' do inc(i);
                 end;
             end;
         end;
     end;
   while param<>'' do
     begin
       if pos(' ',param)=0 then
         begin
           s:=param;
           param:='';
         end
       else
         begin
           s:=copy(param, 1, pos(' ',param)-1);
           delete(param, 1, pos(' ',param));
         end;
       if s='+' then
         begin
           i1:=PopRealStek(st);
           i2:=PopRealStek(st);
           PushRealStek(st, i1+i2);
         end
       else if s='*' then
         begin
           i1:=PopRealStek(st);
           i2:=PopRealStek(st);
           PushRealStek(st, i1*i2);
         end
       else if s='/' then
         begin
           i2:=PopRealStek(st);
           i1:=PopRealStek(st);
           if i2=0 then
             begin
               fatalError:=true;
               errorStr:='Dilennia na nul (division by zero)';
               PushRealStek(st, 0);
             end
           else
             PushRealStek(st, i1/i2);
         end
       else if s='-' then
         begin
           i2:=PopRealStek(st);
           i1:=PopRealStek(st);
           PushRealStek(st, i1-i2);
         end
       else if s='=' then
         begin
           i1:=PopRealStek(st);
           i2:=PopRealStek(st);
           if i1=i2 then
              PushRealStek(st, 1)
           else
              PushRealStek(st, 0);
         end
       else if s='>=' then
         begin
           i2:=PopRealStek(st);
           i1:=PopRealStek(st);
           if i1>=i2 then
              PushRealStek(st, 1)
           else
              PushRealStek(st, 0);
         end
       else if s='<=' then
         begin
           i2:=PopRealStek(st);
           i1:=PopRealStek(st);
           if i1<=i2 then
              PushRealStek(st, 1)
           else
              PushRealStek(st, 0);
         end
       else if s='>' then
         begin
           i2:=PopRealStek(st);
           i1:=PopRealStek(st);
           if i1>i2 then
              PushRealStek(st, 1)
           else
              PushRealStek(st, 0);
         end
       else if s='<' then
         begin
           i2:=PopRealStek(st);
           i1:=PopRealStek(st);
           if i1<i2 then
              PushRealStek(st, 1)
           else
              PushRealStek(st, 0);
         end
       else if s='<>' then
         begin
           i1:=PopRealStek(st);
           i2:=PopRealStek(st);
           if i1<>i2 then
              PushRealStek(st, 1)
           else
              PushRealStek(st, 0);
         end
       else
         begin
           if IsIdentyfikator(s) then
              PushRealStek(st, GetAsReal(s))
           else
              PushRealStek(st, StrToFloat(s))
         end;
     end;
   ObchyslytyReal:=PopRealStek(st);
end;

function TInterpProg.ObchyslytyString(param:string):string;
var
   st:TPStringStek;
   s:string;
   i1, i2:string;
   i, kilkist:integer;
   z1,z2:integer;
begin
   st:=nil;
   if pos('[',param)>0 then {Obchyslennia vlojenyh indeksiv masyvu}
     begin
       kilkist:=0;
       i:=0;
       while i<length(param) do
         begin
           inc(i);
           if param[i]='[' then
             begin
               inc(kilkist);
               if kilkist=1 then
                  z1:=i;
             end;
           if param[i]=']' then
             begin
               dec(kilkist);
               if kilkist=0 then
                 begin
                   z2:=i;
                   s:=copy(param, z1+1, z2-z1-1);
                   delete(param, z1+1, z2-z1-1);
                   s:=IntToStr(ObchyslytyInt(s));
                   insert(s, param, z1+1);
                   i:=z1+1;
                   while param[i]<>']' do inc(i);
                 end;
             end;
         end;
     end;
   while param<>'' do
     begin
       if pos(' ',param)=0 then
         begin
           s:=param;
           param:='';
         end
       else
         begin
           s:=copy(param, 1, pos(' ',param)-1);
           delete(param, 1, pos(' ',param));
         end;
       if s='+' then
         begin
           i2:=PopStringStek(st);
           i1:=PopStringStek(st);
           PushStringStek(st, i1+i2);
         end
       else if s='=' then
         begin
           i2:=PopStringStek(st);
           i1:=PopStringStek(st);
           if i1=i2 then
              PushStringStek(st, '1')
           else
              PushStringStek(st, '0')
         end
       else
         begin
           if IsIdentyfikator(s) then
              PushStringStek(st, GetAsString(s))
           else
              PushStringStek(st, GetStringZnachennia(s))
         end;
     end;
   ObchyslytyString:=PopStringStek(st);
end;


function TInterpProg.ObchyslytyBool(param:string):integer;
var
   st:TPStringStek;
   s, s1, s2:string;
   i1, i2:string;
   i, kilkist:integer;
   z1,z2:integer;
   t1,t2:TTypString;
begin
   st:=nil;
   if pos('[',param)>0 then {Obchyslennia vlojenyh indeksiv masyvu}
     begin
       kilkist:=0;
       i:=0;
       while i<length(param) do
         begin
           inc(i);
           if param[i]='[' then
             begin
               inc(kilkist);
               if kilkist=1 then
                  z1:=i;
             end;
           if param[i]=']' then
             begin
               dec(kilkist);
               if kilkist=0 then
                 begin
                   z2:=i;
                   s:=copy(param, z1+1, z2-z1-1);
                   delete(param, z1+1, z2-z1-1);
                   s:=IntToStr(ObchyslytyInt(s));
                   insert(s, param, z1+1);
                   i:=z1+1;
                   while param[i]<>']' do inc(i);
                 end;
             end;
         end;
     end;
   while param<>'' do
     begin
       if pos(' ',param)=0 then
         begin
           s:=param;
           param:='';
         end
       else
         begin
           s:=copy(param, 1, pos(' ',param)-1);
           delete(param, 1, pos(' ',param));
         end;
      {Operaciyi}
       if IsOperation(s) then
         begin
           if (s='not') or (s='-u') then {unarni operaciyi}
             begin
               i1:=PopStringStek(st)+' '+s;
               if GetTypVyrazu(i1)=c_typ_int then
                 PushStringStek(st, IntToStr(ObchyslytyInt(i1)))
               else
                 PushStringStek(st, FloatToStr(ObchyslytyReal(i1)));
             end
           else
             begin
                i2:=PopStringStek(st);
                i1:=PopStringStek(st);
                t1:=GetTypVyrazu(i1);
                t2:=GetTypVyrazu(i2);
                s1:=i1+' '+i2+' '+s;
                if t1=t2 then
                  begin
                    if GetTypVyrazu(i1)=c_typ_int then
                       PushStringStek(st, IntToStr(ObchyslytyInt(s1)))
                    else if GetTypVyrazu(i1)=c_typ_string then
                      begin
                        if IsLogicOperation(s) then
                           PushStringStek(st, ObchyslytyString(s1))
                        else
                           PushStringStek(st, '"'+ObchyslytyString(s1)+'"');
                      end
                    else
                      begin
                       {ॢi  i iﭭ  ⠪,  ॢ ॠ  i   i⥤஢}
                         if IsLogicOperation(s) then
                           begin
                             if ObchyslytyReal(s1)=0 then s2:='0'
                                                     else s2:='1';
                             PushStringStek(st, s2);
                           end
                         else
                           begin
                             PushStringStek(st, FloatToStr(ObchyslytyReal(s1)));
                           end;
                      end;
                  end
                else if ((t1=c_typ_int) and (t2=c_typ_real)) or ((t2=c_typ_int) and (t1=c_typ_real)) then
                  begin
                      if IsLogicOperation(s) then
                           begin
                             str(ObchyslytyReal(s1):2:0, s2);
                             PushStringStek(st, s2);
                           end
                         else
                           begin
                             PushStringStek(st, FloatToStr(ObchyslytyReal(s1)));
                           end;
                  end
                else
                  begin
                    fatalError:=true;
                    errorStr:='Vyrazy rizni po typy (typ of vyrazes not equal)';
                    PushStringStek(st, '0');
                  end;
             end;
         end
       else
         begin
           if IsIdentyfikator(s) then
             begin
               if GetTypVyrazu(s)=c_typ_string then
                  PushStringStek(st, '"'+GetAsString(s)+'"')
               else
                  PushStringStek(st, GetAsString(s));
             end
           else
             PushStringStek(st, s)
         end;
     end;
   ObchyslytyBool:=StrToInt(PopStringStek(st));
end;



{Inicializuye interprytator}
procedure TInterpProg.InitSpysokZminnyh;
begin
  zminniFirst:=nil;
  platformDos:=false;
  fatalError:=false;
  glybynaPidprogram:=0;
  proceduryFirst:=nil;
  potochnyiOperator:=0;
  isDeclarationProcedure:=false;
  waitingForInput:=false;
end;


{true yakshcho zminna zadeklarovana}
function TInterpProg.VarDeclared(name:string):boolean;
var
   p:TPZminnaZapys;
begin
   p:=zminniFirst;
   while p<>nil do
     begin
       if p^.imya^=name then
         begin
           VarDeclared:=true;
           exit;
         end;
       p:=p^.next;
     end;
   VarDeclared:=false;
end;

type
    TIntArray=array [0..65528 div sizeof(integer)] of integer;
    TRealArray=array [0..65528 div sizeof(real)] of real;
    TStringArray=array [0..65528 div sizeof(string)-1] of string;


{povertaye zminnu yak integer}
function TInterpProg.GetAsInt(name:string):integer;
var
   p:TPZminnaZapys;
   z:^TIntArray;
   arrayStr:string;
   posArray:word;
   code:integer;
   classname:string;
begin
   if pos('->',name)>0 then
     begin
       classname:=copy(name, 1, pos('->',name)-1);
       delete(name, 1, pos('->',name)+1);
       classname:=GetAsString(classname);
       name:=classname+'.'+name;
     end;
   posArray:=0;
   if pos('[',name)>0 then
     begin
       arrayStr:=copy(name, pos('[',name), length(name)-pos('[',name)+1);
       if arrayStr[length(arrayStr)]<>']' then
         begin
          errorStr:='Zvertannia do masyvu nevirne (array access not correct).'+name;
          fatalError:=true;
          exit;
         end;
       name:=copy(name, 1, pos('[',name)-1);
       delete(arrayStr, 1, 1);
       delete(arrayStr, length(arrayStr), 1);
       val(arrayStr, posArray, code);
       if code<>0 then
         begin
           posArray:=ObchyslytyInt(arrayStr);{posArray:=GetAsInt(arrayStr);}
           if fatalError then exit;
         end
     end;
   GetAsInt:=0;
   p:=zminniFirst;
   while p<>nil do
     begin
       if p^.imya^=name then
         begin
           z:=p^.znachennia;
           GetAsInt:=z^[posArray];
           exit;
         end;
       p:=p^.next;
     end;
   fatalError:=true;
   errorStr:='Ne moju znaity zminnu. (Name not found) "'+name+'" -GetAsInt';
end;


{povertaye zminnu yak pointer}
function TInterpProg.GetAsPointer(name:string):pointer;
var
   p:TPZminnaZapys;
   z:pointer;
   arrayStr:string;
   posArray:word;
   code:integer;
   classname:string;
begin
   if pos('->',name)>0 then
     begin
       classname:=copy(name, 1, pos('->',name)-1);
       delete(name, 1, pos('->',name)+1);
       classname:=GetAsString(classname);
       name:=classname+'.'+name;
     end;
   posArray:=0;
   {if pos('[',name)>0 then
     begin
       arrayStr:=copy(name, pos('[',name), length(name)-pos('[',name)+1);
       if arrayStr[length(arrayStr)]<>']' then
         begin
          errorStr:='Zvertannia do masyvu nevirne (array access not correct).'+name;
          fatalError:=true;
          exit;
         end;
       name:=copy(name, 1, pos('[',name)-1);
       delete(arrayStr, 1, 1);
       delete(arrayStr, length(arrayStr), 1);
       val(arrayStr, posArray, code);
       if code<>0 then
         begin
           posArray:=ObchyslytyInt(arrayStr);
           if fatalError then exit;
         end
     end;}
   GetAsPointer:=nil;
   p:=zminniFirst;
   while p<>nil do
     begin
       if p^.imya^=name then
         begin
           z:=p^.znachennia;
           GetAsPointer:=z;
           exit;
         end;
       p:=p^.next;
     end;
   fatalError:=true;
   errorStr:='Ne moju znaity zminnu. (Name not found) "'+name+'" -GetAsPointer';
end;

{Dobavliaye zminnu v spysok zminnyh}
procedure TInterpProg.AddVar(imya:TIdentyfikator; typ:string);
var
   typShort:TTypString;
   arrayStr:string;
   rozmir:word;
   kilkistMas:word;
   p:TPZminnaZapys;
   isArray:boolean;
   code:integer;
   classname:string;
   spntr:^string;
begin
   if pos('->',imya)>0 then
     begin
       classname:=copy(imya, 1, pos('->',imya)-1);
       delete(imya, 1, pos('->',imya)+1);
       classname:=GetAsString(classname);
       imya:=classname+'.'+imya;
     end;
   if glybynaPidprogram=0 then
     if VarDeclared(imya) then
       begin
         errorStr:='Identyfikator vje zadeklarovanyi (identifitier redeclared).';
         fatalError:=true;
         exit;
       end;
   kilkistMas:=1;
   if pos('[',typ)>0 then
     begin
       arrayStr:=copy(typ, pos('[',typ), length(typ)-pos('[',typ)+1);
       if arrayStr[length(arrayStr)]<>']' then
         begin
          errorStr:='Masyv zadano nevirno (array declaration noy correct).';
          fatalError:=true;
          exit;
         end;
       typ:=copy(typ, 1, pos('[',typ)-1);
       isArray:=true;
       delete(arrayStr, 1, 1);
       delete(arrayStr, length(arrayStr), 1);
       val(arrayStr, kilkistMas, code);
       if code<>0 then
         begin
           kilkistMas:=GetAsInt(arrayStr);
           if fatalError then exit;
         end
     end;
   rozmir:=0;
   if typ='integer' then
     begin
       typShort:=c_typ_int;
       rozmir:=sizeof(integer);
     end
   else if typ='real' then
     begin
       typShort:=c_typ_real;
       rozmir:=sizeof(real);
     end
   else if typ='string' then
     begin
       typShort:=c_typ_string;
       rozmir:=sizeof(string);
     end
   else if typ='pointer' then
     begin
       typShort:=c_typ_pointer;
       rozmir:=0;
     end
   else if typ='textfile' then
     begin
       typShort:=c_typ_textfile;
       rozmir:=sizeof(text);
     end
   else if pos('class ',typ)=1 then
     begin
       typShort:=c_typ_class;
       rozmir:=sizeof(string);
       classname:=typ;
       delete(classname,1, 6); {kill class}
       OperatorGosub('gosub '+classname+'.Constructor("'+imya+'");');
     end
   else
     begin
       fatalError:=true;
       errorStr:='Nevidomyi typ (unknown type) "'+typ+'"';
       exit;
     end;
   if rozmir*kilkistMas>65528 then
     begin
       fatalError:=true;
       errorStr:='Zavelykyi rozmir masyvu (Array length is to large).';
       exit;
     end;
   if sizeof(TZminnaZapys)+rozmir*kilkistMas+length(imya)+10241>MaxAvail then
     begin
       fatalError:=true;
       errorStr:='Nemaye pamyati dlia zminnoyi "'+imya+'"';
       exit;
     end;
   GetMem(p, sizeof(TZminnaZapys));{VydilytyPamyatZminnu(p, sizeof(TZminnaZapys));}{GetMem(p, sizeof(TPZminnaZapys));}
   if fatalError then exit;
   p^.next:=zminniFirst;
   zminniFirst:=p;
   GetMem(p^.imya, length(imya)+1);
   p^.imya^:=imya;
   p^.typ:=typShort;
   if typShort<>c_typ_pointer then
       p^.rozmir:=kilkistMas
   else
       p^.rozmir:=0;
   if rozmir<>0 then
     GetMem(p^.znachennia, rozmir*kilkistMas);{VydilytyPamyat(p^.znachennia, rozmir*kilkistMas);}
   if typShort=c_typ_class then
     begin
       spntr:=p^.znachennia;
       spntr^:=classname;
     end;
end;

{ i  祭  ᨫ  i i (var  楤  㭪i)}
procedure TInterpProg.AddVarLink(imya:TIdentyfikator; typ:string; linkimya:TIdentyfikator);
var
   p1,p2:TPZminnaZapys;
begin
   AddVar(imya, 'pointer');
   if fatalError then exit;
   p1:=zminniFirst;
   while p1<>nil do
     begin
       if p1^.imya^=imya then break;
       p1:=p1^.next;
     end;
   p2:=zminniFirst;
   while p2<>nil do
     begin
       if p2^.imya^=linkimya then break;
       p2:=p2^.next;
     end;
   if (p2=nil) or (p1=nil) then
     begin
       fatalError:=true;
       errorStr:='Zminnu ne znaideno "'+linkimya+'"';
       exit;
     end;
   p1^.znachennia:=p2^.znachennia;
   p1^.typ:=p2^.typ;
end;


{Vydaliaye zminnu z spysku zminnyh}
procedure TInterpProg.KillVar(name:TIdentyfikator);
var
   p1,p2:TPZminnaZapys;
   rozmir:word;
begin
   rozmir:=0;
   p1:=zminniFirst;
   p2:=nil;
   while p1<>nil do
     begin
       if p1^.imya^=name then
         begin
           if p2=nil then
             begin
               zminniFirst:=p1^.next;
             end
           else
             begin
               p2^.next:=p1^.next;
             end;
           if p1^.typ='s' then rozmir:=sizeof(string)
            else if p1^.typ='r' then rozmir:=sizeof(real)
            else if p1^.typ='i' then rozmir:=sizeof(integer)
            else if p1^.typ='t' then rozmir:=sizeof(text)
            else if p1^.typ='f' then rozmir:=sizeof(text)
            else if p1^.typ=c_typ_class then rozmir:=sizeof(string);
           if p1^.rozmir>0 then FreeMem(p1^.znachennia, rozmir*p1^.rozmir);
           FreeMem(p1^.imya, length(p1^.imya^)+1);
           FreeMem(p1, sizeof(TZminnaZapys));
           exit;
         end;
       p2:=p1;
       p1:=p1^.next;
     end;
   fatalError:=true;
   errorStr:='Zminna ne isnuye (name not found) "'+name+'"';
end;

procedure TInterpProg.AddProcedure(name:string;params:string);
var
   p:TPProcedureZapys;
begin
   if sizeof(TProcedureZapys)+length(name)+length(params)+10242>MaxAvail then
     begin
       fatalError:=true;
       errorStr:='Nemaye pamyati dlia procedury "'+name+'"';
       exit;
     end;
   GetMem(p, SizeOf(TProcedureZapys));
   GetMem(p^.nazva, length(name)+1);
   GetMem(p^.parametry, length(params)+1);
   p^.nazva^:=name;
   p^.parametry^:=params;
   p^.next:=proceduryFirst;
   p^.pozyciya:=potochnyiOperator;
   proceduryFirst:=p;
end;


function TInterpProg.GetProcParameters(name:string):string;
var
   p:TPProcedureZapys;
begin
   p:=proceduryFirst;
   while p<>nil do
     begin
       if p^.nazva^=name then
         begin
            GetProcParameters:=p^.parametry^;
            exit;
         end;
       p:=p^.next;
     end;
   GetProcParameters:='___';
   fatalError:=true;
   errorStr:='Imya procedury ne znaideno "'+name+'"';
   p^.nazva^:=name;
end;

function TInterpProg.GetProcLine(name:string):integer;
var
   p:TPProcedureZapys;
begin
   p:=proceduryFirst;
   while p<>nil do
     begin
       if p^.nazva^=name then
         begin
            GetProcLine:=p^.pozyciya;
            exit;
         end;
       p:=p^.next;
     end;
   GetProcLine:=0;
   fatalError:=true;
   errorStr:='Imya procedury ne znaideno "'+name+'"';
   p^.nazva^:=name;
end;


procedure TInterpProg.KillSpysokZminnyh;
var
   p:TPZminnaZapys;
begin
   p:=ZminniFirst;
   while p<>nil do
     begin
      KillVar(p^.imya^);
      p:=ZminniFirst;
     end;
end;


{povertaye zminnu yak real}
function TInterpProg.GetAsReal(name:TIdentyfikator):real;
var
   p:TPZminnaZapys;
   z:^TRealArray;
   zi:^TIntArray;
   posArray:word;
   code:integer;
   arrayStr:string;
   classname:string;
begin
   if pos('->',name)>0 then
     begin
       classname:=copy(name, 1, pos('->',name)-1);
       delete(name, 1, pos('->',name)+1);
       classname:=GetAsString(classname);
       name:=classname+'.'+name;
     end;
   posArray:=0;
   if pos('[',name)>0 then
     begin
       arrayStr:=copy(name, pos('[',name), length(name)-pos('[',name)+1);
       if arrayStr[length(arrayStr)]<>']' then
         begin
          errorStr:='Zvertannia do masyvu nevirne (array access not correct).';
          fatalError:=true;
          exit;
         end;
       name:=copy(name, 1, pos('[',name)-1);
       delete(arrayStr, 1, 1);
       delete(arrayStr, length(arrayStr), 1);
       val(arrayStr, posArray, code);
       if code<>0 then
         begin
           posArray:=GetAsInt(arrayStr);
           if fatalError then exit;
         end
     end;
   GetAsReal:=0;
   p:=zminniFirst;
   while p<>nil do
     begin
       if p^.imya^=name then
         begin
           if p^.typ='i' then
             begin
               zi:=p^.znachennia;
               GetAsReal:=zi^[posArray];
               exit;
             end
           else
             begin
               z:=p^.znachennia;
               GetAsReal:=z^[posArray];
               exit;
             end;
         end;
       p:=p^.next;
     end;
   fatalError:=true;
   errorStr:='Ne moju znaity zminnu. (Name not found) "'+name+'" -getasreal';
end;

function TInterpProg.GetType(name:TIdentyfikator):TTypString;
var
   p:TPZminnaZapys;
   z:^TRealArray;
   zi:^TIntArray;
   posArray:word;
   code:integer;
   arrayStr:string;
begin
   if pos('[',name)>0 then
     begin
       name:=copy(name, 1, pos('[', name)-1);
     end;
   GetType:='';
   p:=zminniFirst;
   while p<>nil do
     begin
       if p^.imya^=name then
         begin
           GetType:=p^.typ;
           exit;
         end;
       p:=p^.next;
     end;
   fatalError:=true;
   errorStr:='Ne moju znaity zminnu. (Name not found) "'+name+'" -gettype';
end;

function TInterpProg.GetTypVyrazu(name:string):TTypString;
begin
  if IsIdentyfikator(name) then
     GetTypVyrazu:=GetType(name)
  else
     GetTypVyrazu:=GetConstType(name);
end;

{povertaye zminnu yak string}
function TInterpProg.GetAsString(name:TIdentyfikator):string;
var
   p:TPZminnaZapys;
   zs:^TStringArray;
   zr:^TRealArray;
   zi:^TIntArray;
   s:string;
   posArray:word;
   arrayStr:string;
   code:integer;
   classname:string;
begin
   if pos('->',name)>0 then
     begin
       classname:=copy(name, 1, pos('->',name)-1);
       delete(name, 1, pos('->',name)+1);
       classname:=GetAsString(classname);
       name:=classname+'.'+name;
     end;
   posArray:=0;
   if pos('[',name)>0 then
     begin
       arrayStr:=copy(name, pos('[',name), length(name)-pos('[',name)+1);
       if arrayStr[length(arrayStr)]<>']' then
         begin
          errorStr:='Zvertannia do masyvu nevirne (array access not correct).';
          fatalError:=true;
          exit;
         end;
       name:=copy(name, 1, pos('[',name)-1);
       delete(arrayStr, 1, 1);
       delete(arrayStr, length(arrayStr), 1);
       val(arrayStr, posArray, code);
       if code<>0 then
         begin
           posArray:=GetAsInt(arrayStr);
           if fatalError then exit;
         end
     end;
   GetAsString:='';
   p:=zminniFirst;
   while p<>nil do
     begin
       if p^.imya^=name then
         begin
           if (p^.typ='s') or (p^.typ=c_typ_class) then
             begin
               zs:=p^.znachennia;
               s:=zs^[posArray];
               GetAsString:=s;
               exit;
             end
           else if p^.typ='r' then
             begin
               zr:=p^.znachennia;
               {str(zr^[posArray], s);}
               s:=FloatToStr(zr^[posArray]);
               GetAsString:=s;
               exit;
             end
           else if p^.typ='i' then
             begin
               zi:=p^.znachennia;
               str(zi^[posArray], s);
               GetAsString:=s;
               exit;
             end
           else
             begin
               fatalError:=true;
               errorStr:='Nemojlyva konvertaciya dlia ciogo typu (Can''t convert for thys type) "'+name+'"';
               exit;
             end;
         end;
       p:=p^.next;
     end;
   fatalError:=true;
   errorStr:='Ne moju znaity zminnu. (Name not found) "'+name+'" -getasstring';
end;


{vstanovliuye znachennia zminnoyi}
procedure TInterpProg.SetZminna(name:TIdentyfikator; param:string);
var
   p:TPZminnaZapys;
   zi:^TIntArray;
   i, code:integer;
   r:real;
   zr:^TRealArray;
   s:string;
   zs:^TStringArray;
   posArray:word;
   arrayStr:string;
begin
   posArray:=0;
   if pos('[',name)>0 then
     begin
       arrayStr:=copy(name, pos('[',name), length(name)-pos('[',name)+1);
       if arrayStr[length(arrayStr)]<>']' then
         begin
          errorStr:='Zvertannia do masyvu nevirne (array access not correct).';
          fatalError:=true;
          exit;
         end;
       name:=copy(name, 1, pos('[',name)-1);
       delete(arrayStr, 1, 1);
       delete(arrayStr, length(arrayStr), 1);
       val(arrayStr, posArray, code);
       if code<>0 then
         begin
           posArray:=GetAsInt(arrayStr);
           if fatalError then exit;
         end
     end;
   p:=zminniFirst;
   while p<>nil do
     begin
       if p^.imya^=name then
         begin
           if p^.typ='i' then
             begin
                val(param, i, code);
                if code<>0 then
                   begin
                     if isIdentyfikator(param) then
                        i:=GetAsInt(param)
                     else
                        begin
                          fatalError:=true;
                          errorStr:='"'+param+'" ne ye cilym znachenniam (is not integer).';
                          exit;
                        end;
                   end;
                 zi:=p^.znachennia;
                 zi^[posArray]:=i;
                 exit;
             end
           else if p^.typ='r' then
             begin
                val(param, r, code);
                if code<>0 then
                   begin
                     if isIdentyfikator(param) then
                        r:=GetAsReal(param)
                     else
                        begin
                          fatalError:=true;
                          errorStr:='"'+param+'" ne ye diysnym znachenniam (is not float).';
                          exit;
                        end;
                   end;
                 zr:=p^.znachennia;
                 zr^[posArray]:=r;
                 exit;
             end
           else if p^.typ='s' then
             begin
                 s:=param;
                 if (param[1]<>'"') or (param[length(param)]<>'"') then
                   begin
                     if isIdentyfikator(param) then
                        s:=GetAsString(param)
                     else
                        begin
                          fatalError:=true;
                          errorStr:='"'+param+'" ne ye strichkoyu (is not string).';
                          exit;
                        end;
                   end
                 else
                   begin
                     if param='""' then s:=''
                      else s:=GetStringZnachennia(param);
                   end;
                 zs:=p^.znachennia;
                 zs^[posArray]:=s;
                 exit;
             end;
         end;
       p:=p^.next;
     end;
   fatalError:=true;
   errorStr:='Ne moju znaity zminnu. (Name not found) "'+name+'" -setzminna';
end;


{vstanovliuye znachennia zminnoyi yak pointer}
procedure TInterpProg.SetZminnaAsPointer(name:TIdentyfikator; param:pointer);
var
   p:TPZminnaZapys;
begin
   p:=zminniFirst;
   while p<>nil do
     begin
       if p^.imya^=name then
         begin
           p^.znachennia:=param;
         end;
       p:=p^.next;
     end;
   fatalError:=true;
   errorStr:='Ne moju znaity zminnu. (Name not found) "'+name+'" -setzminna';
end;


{Operator prysvoyennia let <zminna>=<znachennia>;}
procedure TInterpProg.OperatorLet(param:string);
var
   zminna:string;
   znachennia:string;
   operaciya:string;
   i:integer;
   s1:TTypString;
begin
   delete(param, 1, 4); {kill let}
   if (pos('=', param)=0) and (pos('++', param)=0) and (pos('--', param)=0) then
     begin
        fatalError:=true;
        errorStr:='Vidsutniy symvol (symbol expected) "="';
        exit;
     end;
   if (pos('=', param)<>0) then operaciya:='='
    else if (pos('++', param)<>0) then operaciya:='++'
     else operaciya:='--';
   zminna:=copy(param, 1, pos(operaciya,param)-1);
   if operaciya<>'=' then
     begin
       i:=GetAsInt(zminna);
       if operaciya='++' then
           inc(i)
       else
           dec(i);
       Str(i, znachennia);
       SetZminna(zminna, znachennia);
       exit;
     end;
   delete(param, 1, pos('=',param));
   znachennia:=copy(param, 1, pos(';',param)-1);
   if znachennia='' then znachennia:='"0"';
   s1:=GetType(zminna);
   if s1='i' then
     znachennia:=IntToStr(ObchyslytyInt(znachennia))
   else if s1='r' then
     znachennia:=FloatToStr(ObchyslytyReal(znachennia))
   else if s1='s' then
     znachennia:='"'+ObchyslytyString(znachennia)+'"'
   else
     begin
       {Tut maye buty shche shchos, ale chy bude?}
     end;
   SetZminna(zminna, znachennia);
end;

{Operator obyavlennia prostoyi zminnoyi   var <nazva>:<typ>; }
procedure TInterpProg.OperatorVar(param:string);
var
   nazva:TIdentyfikator;
   typ:string;
begin
   delete(param, 1, 4); {kill var}
   if (pos(':', param)=0) then
     begin
        fatalError:=true;
        errorStr:='Vidsutniy symvol (symbol expected) ":"';
        exit;
     end;
   nazva:=copy(param, 1, pos(':',param)-1);
   delete(param, 1, pos(':',param));
   typ:=copy(param, 1, pos(';',param)-1);
   AddVar(nazva, typ);
end;

{Operator obyavlennia klasovoyi zminnoyi   varclass <nazva_klasu> <nazva>:<typ>;
Stvoriuye zminnu value(<nazva_klasu>).<nazva>}
procedure TInterpProg.OperatorVarClass(param:string);
var
   nazva:string;
   nazva_klasu:string;
   typ:string;
begin
   delete(param, 1, 9); {kill varclass}
   if (pos(' ', param)=0) then
     begin
        fatalError:=true;
        errorStr:='Vidsutniy symvol (symbol expected) " " - varclass';
        exit;
     end
   else
     begin
        nazva_klasu:=copy(param, 1, pos(' ',param)-1);
        delete(param, 1, pos(' ',param));
        nazva_klasu:=GetAsString(nazva_klasu);
     end;
   if (pos(':', param)=0) then
     begin
        fatalError:=true;
        errorStr:='Vidsutniy symvol (symbol expected) ":"';
        exit;
     end;
   nazva:=copy(param, 1, pos(':',param)-1);
   delete(param, 1, pos(':',param));
   typ:=copy(param, 1, pos(';',param)-1);
   AddVar(nazva_klasu+'.'+nazva, typ);
end;


{Operator vydalennia klasovoyi zminnoyi   killvarclass <nazva_klasu> <nazva>;
ubyvaye zminnu value(<nazva_klasu>).<nazva>}
procedure TInterpProg.OperatorKillVarClass(param:string);
var
   nazva:string;
   nazva_klasu:string;
   typ:string;
begin
   delete(param, 1, 13); {kill killvarclass}
   if (pos(' ', param)=0) then
     begin
        fatalError:=true;
        errorStr:='Vidsutniy symvol (symbol expected) " " - killvarclass';
        exit;
     end
   else
     begin
        nazva_klasu:=copy(param, 1, pos(' ',param)-1);
        delete(param, 1, pos(' ',param));
        nazva_klasu:=GetAsString(nazva_klasu);
     end;
   nazva:=copy(param, 1, pos(';',param)-1);
   KillVar(nazva_klasu+'.'+nazva);
end;



{
drukuye zminnu println <var>;      ---writeln
drukuye zminnu print <var>;  ---write
}
procedure TInterpProg.OperatorPrintVal(param:string; lin:boolean);
var
   name:TIdentyfikator;
   s:string;
   x:real;
begin
   if lin then
      delete(param, 1, 8)        {kill println}
   else
      delete(param, 1, 6);      {kill print}
   s:=copy(param, 1, pos(';',param)-1);
   name:=s;
   if IsIdentyfikator(name) then
      s:=GetAsString(name)
   else
     begin
       if pos('"',s)>0 then
         begin
           s:=GetStringZnachennia(s);
         end;
     end;
   s:=ConvertFromSpecialChars(s);
   if lin then
      OSDrukParamLn(s)
   else
      OSDrukParamNoLn(s)
end;

{input <poyasnennia>,<zminna>;}
procedure TInterpProg.OperatorInput(param:string);
var
   i:integer;
   s:string;
   st:string;
begin
   delete(param, 1, 6);        {kill input}
   param:=copy(param, 1, length(param)-1);
   s:='';
   if param='' then
      begin
        fatalError:=true;
        errorStr:='Syntaksychna pomylka (syntax error)  ->INPUT';
        exit;
      end;
   for i:=length(param) downto 1 do
      begin
        if param[i]=',' then
           begin
              delete(param, i, length(param)-i+1);
              break;
           end;
        s:=param[i]+s;
      end;
   param:=GetStringZnachennia(param);
   st:=GetType(s);
   waitingForInput:=true;
   osInputVarName:=s;
   OSInputBegin(param);
   if platformDos then EndOfOperatorInput;
{   if st=c_typ_string then
     SetZminna(s, '"'+ConvertToSpecialChars(OSInput(ConvertFromSpecialChars(param)))+'"')
   else
     SetZminna(s, OSInput(param));}
end;

procedure TInterpProg.EndOfOperatorInput;
var
   st:string;
begin
   if platformDos then resultInput:=OSInputEnd;
   st:=GetType(osInputVarName);
   if st=c_typ_string then
     SetZminna(osInputVarName, '"'+ConvertToSpecialChars(resultInput)+'"')
   else
     SetZminna(osInputVarName, resultInput);
end;


{vbyvaye zminnu killvar <var>;}
procedure TInterpProg.OperatorKillVar(param:string);
var
   name:TIdentyfikator;
   s:string;
begin
   delete(param, 1, 8); {kill killvar}
   name:=copy(param, 1, pos(';',param)-1);
   KillVar(name);
end;



procedure TInterpProg.OperatorGoto(param:string);
var
   s:string;
   i:longint;
   labelPos:longint;
   code:integer;
begin
   delete(param, 1, 5); {kill killvar}
   param:=copy(param, 1, length(param)-1);
   if IsIdentyfikator(param) and (param[1]<>'+') and (param[1]<>'-') then
     begin
        reset(f);
        potochNyiOperator:=0;
        while not eof(f) do
           begin
             readln(f,s);
             inc(potochNyiOperator);
             if s='label '+param+';' then exit;
           end;
        fatalError:=true;
        errorStr:='Mitku ne znaideno (Label not found)  "'+param+'"';
     end
   else
     begin
        reset(f);
        if (param[1]<>'+') and (param[1]<>'-') then
           labelPos:=StrToInt(param)
        else
           begin
              if param[1]='+' then
                begin
                  delete(param,1,1);
                  labelPos:=StrToInt(param)+potochnyiOperator;
                end
              else if param[1]='-' then
                begin
                  delete(param,1,1);
                  labelPos:=potochnyiOperator-StrToInt(param);
                end
           end;
        i:=0;
        while not eof(f) do
           begin
             inc(i);
             if i>=labelPos then
               begin
                 potochNyiOperator:=labelPos-1;
                 exit;
               end;
             readln(f,s);
           end;
        fatalError:=true;
        errorStr:='Pozyciynu mitku ne znaideno (Label not found)  "'+param+'"';
     end;
end;

{operator obyavlennia procedury
procedure <name>(<param1>:<typ1>...);}
procedure TInterpProg.OperatorProcedure(param:string);
var
   name:TIdentyfikator;
   parametry:string;
   s:string;
begin
   delete(param, 1, 10); {kill procedure}
   name:=copy(param, 1, pos('(',param)-1);
   delete(param, 1, pos('(',param));
   parametry:=copy(param, 1, pos(')',param)-1);
   AddProcedure(name, parametry);
   isDeclarationProcedure:=true;
end;

{
vyklyk procedury
}
procedure TInterpProg.OperatorGosub(param:string);
var
   nameproc:string;
   faktParam:string;
   formParam:string;
   fakt:string;
   form:string;
   zamovch:string;
   znach:string;
   s:string;
   lineproc:integer;
   classname:string;
   isVarInForm:boolean;
   standartProcID:integer;
begin
   if pos('(',param)=0 then
     begin
       fatalError:=true;
       errorStr:='Vidsutniy symvol "(" - gosub';
       exit;
     end;
   delete(param, 1, 6); {kill gosub}
   if (pos('.',param)>0) and (pos('.Constructor',param)=0) and (pos('.Destructor',param)=0) then
     begin
       classname:='';
       {writeln('param=',param,'  class=',classname);}
       while (pos('.',param)>0) and (pos('.',param)<pos('(',param)) do
         begin
           classname:=copy(param, 1, pos('.',param)-1);
           delete(param, 1, pos('.',param));
           if (pos('.',param)>0) and (pos('.',param)<pos('(',param)) then
             classname:=classname+'.';
           writeln('param=',param,'  class=',classname);
         end;
       {readln;}
       nameproc:=GetAsString(classname)+'.'+copy(param, 1, pos('(',param)-1);
       {writeln(nameproc);}
     end
   else
     nameproc:=copy(param, 1, pos('(',param)-1);
   standartProcID:=IsStandProc(nameproc);
   delete(param, 1, pos('(',param));
   faktParam:=copy(param, 1, pos(')',param)-1);
   if standartProcID>=0 then
       formParam:=GetProcParameters(nameproc)
   else
       formParam:=GetStProcParameters(standartProcID);
   if fatalError then exit;
   inc(glybynaPidprogram);
   if (pos('.', nameproc)>0) and (pos('.Constructor',nameproc)=0) and (pos('.Destructor',nameproc)=0)  then
     begin
       AddVar('self', 'string');
       OperatorLet('let self="'+classname+'";');
     end;
   while formParam<>'' do
     begin
       isVarInForm:=false;
       zamovch:='';
       if pos(';',formParam)=0 then
         begin
           form:=formParam;
           formParam:='';
         end
       else
         begin
           form:=copy(formParam, 1, pos(';', formParam)-1);
           delete(formParam, 1, pos(';', formParam));
         end;
       if pos('=',form)>0 then
         begin
           zamovch:=form;
           form:=copy(form,1, pos('=',zamovch)-1);
           delete(zamovch, 1, pos('=',zamovch));
         end;
       if pos('var ', form)=1 then
         begin
           delete(form, 1, 4);
           isVarInForm:=true;
         end;
       if pos(',',faktParam)=0 then
         begin
           fakt:=faktParam;
           faktParam:='';
         end
       else
         begin
           fakt:=copy(faktParam, 1, pos(',', faktParam)-1);
           delete(faktParam, 1, pos(',', faktParam));
         end;
       if not isVarInForm then
          OperatorVar('var '+form+';');
       if pos(':', form)>0 then
         begin
           form:=copy(form, 1, pos(':', form)-1);
         end
       else
         begin
           fatalError:=true;
           errorStr:='Nevirno opysani parametry v proceduri "'+nameproc+'"';;
           exit;
         end;
       if isVarInForm then
         begin
            AddVarLink(form, c_typ_pointer_full, fakt);
         end
       else
         begin
           if fakt='' then
              znach:=zamovch
           else
              znach:=fakt;
           OperatorLet('let '+form+'='+znach+';');
         end;
     end;
   {writeln('standartProcID=',standartProcID);
   readln;}
   if standartProcID>=0 then
     begin
        lineproc:=GetProcLine(nameproc);
        if fatalError then exit;
        PushLongintStek(returnToStek, potochnyiOperator);
        OperatorGoto('goto '+IntToStr(lineproc+1)+';');
     end
   else
     begin
        RunStandProc(standartProcID, self);
        dec(glybynaPidprogram);
     end;
end;


procedure TInterpProg.OperatorReturn(param:string);
begin
  potochnyiOperator:=PopLongintStek(returnToStek)+1;
  dec(glybynaPidprogram);
  if glybynaPidprogram<0 then
    begin
      fatalError:=true;
      errorStr:='return bez gosub';
    end;
  OperatorGoto('goto '+IntToStr(potochnyiOperator)+';');
end;


{
if val then labelname;  //val:integer <0=false >0=true
}
procedure TInterpProg.OperatorIf(param:string);
var
   umova:string;
   param2:string;
begin
  delete(param, 1, 3); {kill if}
  umova:=copy(param, 1, pos(' then ', param)-1);
  delete(param, 1, pos(' then ', param));
  delete(param, 1, 5);
  if pos(' else ',param)=0 then
    begin
       if ObchyslytyBool(umova)>0 then OperatorGoto('goto '+param);
{       if ObchyslytyInt(umova)>0 then OperatorGoto('goto '+param);}
{       if GetAsInt(umova)>0 then OperatorGoto('goto '+param);}
    end
  else
    begin
       param2:=copy(param, 1, pos(' else ',param)-1);
       delete(param, 1, pos(' else ', param));
       delete(param, 1, 5);
       if ObchyslytyBool(umova)>0 then OperatorGoto('goto '+param2+';')
                     {if GetAsInt(umova)>0 then OperatorGoto('goto '+param2+';')}
       else OperatorGoto('goto '+param)
    end;
end;

{vykonuye naiprostishyi operator}
procedure TInterpProg.ExecOperator(param:string);
var
   z:text;
begin
  inc(potochnyiOperator);
{  assign(z,'term.log');
  append(z);
  writeln(z,'nomer=',potochnyiOperator,'  oper=',param);
  close(z);}
  if isDeclarationProcedure then
    begin    {rejym deklaraciyi procedury}
      if (param='return;') then
        begin
           isDeclarationProcedure:=false
        end;
      exit;
    end;
  if pos('//', param)=1 then
    begin
      {Nichogo ne robymo - komentar}
    end
  else if pos(';', param)=0 then
    begin
      fatalError:=true;
      errorStr:='Vidsutniy symvol (symbol expected)  ";"  '+param;
    end
  else if pos('let ', param)=1 then
    begin
      OperatorLet(param);
    end
  else if pos('var ', param)=1 then
    begin
      OperatorVar(param);
    end
  else if pos('gosub ', param)=1 then
    begin
      OperatorGosub(param);
    end
  else if pos('return;', param)=1 then
    begin
      OperatorReturn(param);
    end
  else if pos('procedure ', param)=1 then
    begin
      OperatorProcedure(param);
    end
  else if pos('println ', param)=1 then
    begin
      OperatorPrintVal(param, true);
      isOutput:=true;
    end
  else if pos('print ', param)=1 then
    begin
      OperatorPrintVal(param, false);
      isOutput:=true;
    end
  else if pos('input ', param)=1 then
    begin
      OperatorInput(param);
    end
  else if pos('killvar ', param)=1 then
    begin
      OperatorKillVar(param);
    end
  else if pos('label ', param)=1 then
    begin
      {Mitka, nichogo ne robyty}
    end
  else if pos('goto ', param)=1 then
    begin
      OperatorGoto(param);
    end
  else if pos('if ', param)=1 then
    begin
      OperatorIf(param);
    end
  else if pos('varclass ', param)=1 then
    begin
      OperatorVarClass(param);
    end
  else if pos('killvarclass ', param)=1 then
    begin
      OperatorKillVarClass(param);
    end
  else if param='halt;' then
    begin
      isRunning:=false;
      halt;
    end
  else
    begin
      fatalError:=true;
      errorStr:='Nevidomyi operator (unknown operator). "'+param+'"';
    end;
  if fatalError then
    begin
      OutPutError;
      fatalError:=false;
      isRunning:=false;
    end;
end;

{run "name";}
procedure TInterpProg.Run(s:string);
var
   oper:string;
begin
  delete(s,1,4);
  delete(s,1,1);
  delete(s,length(s),1);
  delete(s,length(s),1);
  assign(f, s);
  reset(f);
  while not eof(f) do
    begin
      readln(f, oper);
{      writeln('-->', oper, '<--');}
      if oper='' then continue;
      ExecOperator(oper);
    end;
  close(f);
end;

{vyvodyt text pomylky}
procedure TInterpProg.OutPutError;
begin
  OSOutPutError('POMYLKA: '+errorStr);
end;

procedure TInterpProg.InitExecFile(filename:string);
begin
  assign(f, filename);
  reset(f);
  InitSpysokZminnyh;
  isRunning:=true;
  platformDos:=false;
  waitingForInput:=false;
end;


{vykonannia nastupnogo operatora v politivskih programah OnIdle}
procedure TInterpProg.NextOperator;
var
   oper:string;
begin
  if not eof(f) and isRunning then
    begin
      readln(f, oper);
      if oper='' then exit;
      ExecOperator(oper);
    end
  else
    begin
      close(f);
      KillSpysokZminnyh;
      isRunning:=false;
    end;
end;


procedure TInterpProg.ContinueAfterInput(inputVal:string);
begin

end;


end.


{
9.03.05
  realizovano procedury

4.04.05
  killvarclass s1 name; - vydaliaye zminnu VALUE(s1).s2
  rejym deklaraciyi procedury

3.04.05
  Realizovano perevirku vyhodu
      za granyci vilnoyi pamyati pry vydilenni pamyati dlia zminnoyi.
  varclass s1 name:<typ>; - stvoriuye zminnu VALUE(s1).s2
  dodano novyi typ - pointer
  dobavleno proceduru dobavlennia procedury

27.03.05
  Povnistiu realizovano boolean vyrazy v if

20.03.05
  Masyvy v real i string

19.03.05
  Realizovani masyvy v cilyh aryfm vyrazah

18.03.05
  Aryfmetychni vyrazy (polskyi zapys) dlia real and string

17.03.05
  Aryfmetychni vyrazy (polskyi zapys) dlia int

16.03.05
  Perehid na obyekty

14.03.05
  if x then labelname1 else labelname2;
  //komentar

13.03.05
  goto labelname;
  label labelname;
  let x++;
  let x--;
  if x then labelname;

i
  var x:type;
  killvar x;
  let x=val;
  println x;
  print x;
  run "filename";
  input poyasn,x;
  ॠi ᨢ
}
